/*
 * Tagged Collection document for a Document-View editing architecture
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma implementation

#include "config.h"

#include "DebtagsDocument.h"

#include <InputMerger.h>
#include <ItemGrouper.h>
#include <TagcollSerializer.h>
#include <TagcollBuilder.h>
#include <ExpressionFilter.h>
#include <StdioParserInput.h>
#include <StringParserInput.h>

#include "Environment.h"
#include "DebtagsEnvironment.h"

#include <errno.h>

using namespace std;

/*
const char* fn_tagdb = "/var/lib/debtags/package-tags";
const char* fn_impls = "/var/lib/debtags/implications";
const char* fn_deriv = "/var/lib/debtags/derived-tags";
*/

DebtagsDocument::DebtagsDocument() throw () : _undoTail(0)
{
	_fileName = string(getenv("HOME")) + "/." PACKAGE;
}

void DebtagsDocument::do_changed()
{
	_signal_changed.emit();
}

void DebtagsDocument::do_filename_changed()
{
	_signal_filename_changed.emit();
}

bool DebtagsDocument::canUndo() const
{
	return _undoTail > 0;
}

bool DebtagsDocument::canRedo() const
{
	return _undoTail < _undoBuffer.size();
}

void DebtagsDocument::undo()
{
	if (!canUndo())
		return;

	debug("Pre undo: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

	_undoTail--;
	_undoBuffer[_undoTail] = _collection.applyChange(_undoBuffer[_undoTail]);

	debug("Post undo: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

	do_changed();
}

void DebtagsDocument::redo()
{
	if (!canRedo())
		return;

	debug("Pre redo: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

	_undoBuffer[_undoTail] = _collection.applyChange(_undoBuffer[_undoTail]);
	_undoTail++;

	debug("Post redo: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

	do_changed();
}

void DebtagsDocument::applyChange(const TagcollChange<Debtags::Package*, std::string>& change)
{
	TagcollChange<Debtags::Package*, std::string> reverse = _collection.applyChange(change);
	
	if (reverse != change)
	{
		debug("Pre change: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

		// Push the reverse change in the undo buffer
		if (_undoTail == _undoBuffer.size())
		{
			_undoBuffer.push_back(reverse);
			_undoTail++;
		}
		else
		{
			_undoBuffer[_undoTail++] = reverse;
			_undoBuffer.resize(_undoTail);
		}

		debug("Post change: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());
			
		do_changed();
	} else
		debug("Change had no effect\n");
}

/*
template<class T>
void DebtagsDocument<T>::load()
	throw (FileException, ParserException)
{
	//_collection.clear();
	DebtagsEnvironment::get().outputPatched(_collection);

	// Output the merged collection to consumer
	/ *
	TagcollBuilder builder(_handleMaker);
	merger.output(builder);
	_collection = builder.collection();
	* /
	_undoBuffer.clear();
	_undoTail = 0;
	_fileName = "";

	do_filename_changed();
	do_changed();
}
*/

void DebtagsDocument::save() throw (FileException)
{
	_collection.save();
	//DebtagsEnvironment::get().savePatch(_collection);
	/*
	FILE* out = fopen(file.c_str(), "wt");
	if (!out)
		throw FileException(errno, "opening file " + file);
	
	// Output the collection, grouping the items
	TagcollSerializer serializer(out);
	ItemGrouper<string> grouper;
	output(grouper);
	grouper.output(serializer);

	fclose(out);

	if (file != _fileName)
	{
		_fileName = file;
		do_filename_changed();
	}
	*/
}

/*
template<class T>
OpSet<T> DebtagsDocument<T>::handlesToItems(const OpSet<int>& handles) const throw ()
{
	OpSet<T> res;
	for (set<int>::const_iterator i = handles.begin(); i != handles.end(); i++)
		res += _handleMaker.getItem(*i);
	return res;
}
*/

/*
template<class T>
void DebtagsDocument<T>::output(TagcollConsumer<T>& consumer) throw ()
{
	_collection.output(consumer);
	/ *
	for (typename TagCollection<int>::const_iterator i = _collection.begin();
			i != _collection.end(); i++)
		consumer.consume(handlesToItems(i->second), i->first);
	* /
}*/

/*
template<class T>
void DebtagsDocument<T>::output(TagcollConsumer<int>& consumer) throw ()
{
	for (typename TagCollection<int>::const_iterator i = _collection.begin();
			i != _collection.end(); i++)
		consumer.consume(i->second, i->first);
}*/
// vim:set ts=4 sw=4:
