/*
 * A single view on a TagcollDocument
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma implementation

#include "FilterPanel.h"

#include <stringf.h>
#include "Environment.h"

#include "DebtagsDocument.h"
#include "PackageBuffer.h"

#include <map>

#include <gtkmm/table.h>
#include <gtkmm/label.h>
#include <gtkmm/menu.h>
#include <gtkmm/treemodel.h>

using namespace std;

template class FilterPanel<DebtagsDocument>;

template<class DOC>
FilterPanel<DOC>::FilterPanel(DOC& doc)
	: Gtk::Frame("Filter"), doc(doc), tagSelector(doc, pkgs)
{
	Gtk::Table* table = manage(new Gtk::Table(6, 2));
	add(*table);


	// Add the TreeView, inside a ScrolledWindow
	scrolledItemList.add(itemList);
	// Only show the scrollbars when they are necessary:
	scrolledItemList.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	Gtk::Menu* smenu = Gtk::manage(new Gtk::Menu());
	smenu->items().push_back(Gtk::Menu_Helpers::MenuElem("_None"));
	smenu->items().push_back(Gtk::Menu_Helpers::MenuElem("_Installed"));
	smenu->items().push_back(Gtk::Menu_Helpers::MenuElem("N_ot installed"));
	statusFilter.set_menu(*smenu);

	if (char* defmail = getenv("DEBEMAIL"))
	{
		filter.setMaintainer(defmail);
		maintFilter.set_text(defmail);
		statusFilter.set_history(0);
	} else {
		filter.setInstalled(Debtags::Filter::YES);
		statusFilter.set_history(1);
	}

	Gtk::Frame* tagFrame = manage(new Gtk::Frame("Tags:"));
	tagFrame->add(tagSelector);

	table->attach(*manage(new Gtk::Label("Name:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER)), 0, 1, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);
	table->attach(nameFilter, 1, 2, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);
	table->attach(*manage(new Gtk::Label("Text:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER)), 0, 1, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);
	table->attach(ftextFilter, 1, 2, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);
	table->attach(*manage(new Gtk::Label("Maintainer:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER)), 0, 1, 2, 3, Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);
	table->attach(maintFilter, 1, 2, 2, 3, Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);
	table->attach(*manage(new Gtk::Label("Status:", Gtk::ALIGN_RIGHT, Gtk::ALIGN_CENTER)), 0, 1, 3, 4, Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);
	table->attach(statusFilter, 1, 2, 3, 4, Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);
	table->attach(*tagFrame, 0, 2, 4, 5, Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);
	table->attach(scrolledItemList, 0, 2, 5, 6);
	table->attach(foundStats, 0, 2, 6, 7, Gtk::FILL | Gtk::EXPAND, Gtk::SHRINK);

	nameFilter.signal_changed().connect(SigC::slot(*this, &FilterPanel<DOC>::on_nameFilter_changed));
	maintFilter.signal_changed().connect(SigC::slot(*this, &FilterPanel<DOC>::on_maintFilter_changed));
	ftextFilter.signal_changed().connect(SigC::slot(*this, &FilterPanel<DOC>::on_ftextFilter_changed));
	statusFilter.signal_changed().connect(SigC::slot(*this, &FilterPanel<DOC>::on_statusFilter_changed));
	tagSelector.signal_changed().connect(SigC::slot(*this, &FilterPanel<DOC>::on_tagFilter_changed));

	// Create the Tree model
	itemListModel = Gtk::ListStore::create(itemListModelColumns);
	itemList.set_model(itemListModel);

	// Add the view columns
	itemList.append_column("Name", itemListModelColumns.name);
	itemList.append_column("Description", itemListModelColumns.desc);

	itemList.get_column(0)->set_resizable(true);
	itemList.get_column(1)->set_resizable(true);

	Glib::RefPtr<Gtk::TreeSelection> itemListSelection = itemList.get_selection();
	itemListSelection->set_mode(Gtk::SELECTION_SINGLE);

	itemListSelection->signal_changed().connect(SigC::slot(*this, &FilterPanel<DOC>::on_selection_changed));
	
	//itemList.add_events(Gdk::BUTTON_PRESS_MASK);
	//itemList.signal_event().connect(SigC::slot(*this, &ItemList::on_event));

	//tagSelector.signal_add_tag().connect(SigC::slot(*this, &FilterPanel<DOC>::on_signal_add_tag));
	//tagSelector.signal_remove_tag().connect(SigC::slot(*this, &FilterPanel<DOC>::on_signal_remove_tag));
	//itemList.signal_focus_in_event().connect(SigC::slot(*this, &FilterPanel::do_list_focus_in_event));
	updateList();
}

template<class DOC>
void FilterPanel<DOC>::on_statusFilter_changed() throw ()
{
	int cur = statusFilter.get_history();
	filter.setInstalled(cur == 0 ? Debtags::Filter::ANY : cur == 1 ? Debtags::Filter::YES : Debtags::Filter::NO);
	updateList();
}

template<class DOC>
void FilterPanel<DOC>::on_tagFilter_changed() throw ()
{
	filter.setDebtags(tagSelector.selected());
	updateList();
}

template<class DOC>
void FilterPanel<DOC>::on_selection_changed() throw ()
{
	Glib::RefPtr<Gtk::TreeSelection> sel = itemList.get_selection();
	Gtk::TreeModel::iterator iter = sel->get_selected();
	if (iter)
	{
		Gtk::TreeModel::Row row = *iter;
		Debtags::Package* p = row[itemListModelColumns.pkg];
		fprintf(stderr, "Selected %.*s\n", PFSTR(p->name()));
		signal_selected().emit(p);
	}
}

template<class DOC>
void FilterPanel<DOC>::updateList() throw ()
{
	static const unsigned int max_pkg = 200;
	printf("UPDATELIST start\n");
	pkgs.clear();

	doc.collection().output(pkgs, filter);
	PackageBuffer::iterator lastIter = pkgs.size() > max_pkg ? pkgs.begin() + max_pkg : pkgs.end();
	
	string state;
	if (lastIter != pkgs.end())
	{
		state = stringf::fmt(">%d (%d) packages found", max_pkg, pkgs.size() + pkgs.discarded());
		//printf("The limit of %d visualized packages has been reached: please narrow your search.\n", max_pkg);
	} else {
		state = stringf::fmt("%d packages found", pkgs.size());
	}
	foundStats.set_text(state);

	pkgs.sort();

	printf("UPDATELIST populate\n");
	Gtk::TreeModel::Row row;
	itemListModel->clear();
	for (PackageBuffer::iterator i = pkgs.begin(); i != lastIter; i++)
	{
		row = *(itemListModel->append());
		row[itemListModelColumns.pkg] = *i;
		row[itemListModelColumns.name] = (*i)->name();
		row[itemListModelColumns.desc] = (*i)->desc();
		//if (sel.find(*i) != sel.end())
			//itemList.get_selection()->select(row);
	}
	printf("UPDATELIST end\n");
}

/*
template<class DOC>
void FilterPanel<DOC>::on_signal_add_tag(std::string tag)
{
	// Get the contents of the selection 
	std::map< string, OpSet<string> > data = getSelection();
	if (data.empty())
		data = getAllItems();
	if (data.empty())
		return;

	// Prepare the change
	TagCollection<int>::Change change;
	for (std::map< string, OpSet<string> >::const_iterator i = data.begin();
			i != data.end(); i++)
		change.insert(make_pair(doc.handles().getHandle(i->first), i->second + tag));

	// Make the change
	doc.applyChange(change);

	signal_request_tagcoll_change().emit(change);
}

template<class DOC>
void FilterPanel<DOC>::on_signal_remove_tag(std::string tag)
{
	// Get the contents of the selection 
	std::map< string, OpSet<string> > data = getSelection();
	if (data.empty())
		data = getAllItems();
	if (data.empty())
		return;
	
	// Prepare the change
	TagCollection<int>::Change change;
	for (std::map< string, OpSet<string> >::const_iterator i = data.begin();
			i != data.end(); i++)
		change.insert(make_pair(doc.handles().getHandle(i->first), i->second - tag));

	// Make the change
	doc.applyChange(change);

	signal_request_tagcoll_change().emit(change);
}

*/



/*
bool FilterPanel::do_list_focus_in_event(GdkEventFocus*)
{
	warning("LFIE\n");
	return false;
}
*/

// vim:set ts=4 sw=4:
